/**
 * OW2 Util
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: IRecorderAccessManager.java 4601 2009-02-16 09:35:38Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.pool.impl.enhanced.api.recorder.accesmanager;

import java.util.List;

import org.ow2.util.pool.impl.enhanced.api.basic.accessmanager.IAccessManager;
import org.ow2.util.pool.impl.enhanced.api.recorder.IRecorderPoolItem;
import org.ow2.util.pool.impl.enhanced.api.recorder.RecorderPoolItemState;

/**
 *
 * @author Gael Lalire
 */
public interface IRecorderAccessManager<E, I> extends IAccessManager<IRecorderPoolItem<E, I>> {

    int NO_POOL_ITEM_TO_RECORD = -1;

    int NO_POOL_ITEM_TO_FETCH = -1;

    /**
     * @param poolItemList the list of pool item
     * @return a value between 0 and poolItemList.size()-1 or
     * NO_POOL_ITEM_TO_RECORD if you do not want to record any item
     */
    int choosePoolItemToRecord(List<? extends IRecorderPoolItem<E, I>> poolItemList);

    /**
     * @param poolItemList the list of pool item
     * @return a value between 0 and poolItemList.size()-1 or
     * NO_POOL_ITEM_TO_FETCH if you do not want to fetch any item
     */
    int choosePoolItemToFetch(List<? extends IRecorderPoolItem<E, I>> poolItemList);

    /**
     * The poolItem was previously removed from poolItemList so you need to choose a new position.
     * @param poolItemList the list of pool item
     * @param poolItem the recorded poolItem
     * @param previousState the previous state of item (to get new state use poolItem.getState())
     * @return a value between 0 and poolItemList.size()
     */
    int putPoolItemWhoseStateChanged(List<? extends IRecorderPoolItem<E, I>> poolItemList, IRecorderPoolItem<E, I> poolItem,
            RecorderPoolItemState previousState);

}
