/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: BasicPoolFactory.java 4601 2009-02-16 09:35:38Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.pool.impl.enhanced.impl.basic;

import org.ow2.util.pool.impl.enhanced.api.IPoolItemFactory;
import org.ow2.util.pool.impl.enhanced.api.basic.IBasicPool;
import org.ow2.util.pool.impl.enhanced.api.basic.IBasicPoolFactory;
import org.ow2.util.pool.impl.enhanced.api.basic.accessmanager.IAccessManager;
import org.ow2.util.pool.impl.enhanced.impl.basic.accessmanager.LastUsedAccessManager;

/**
 *
 * @author Gael Lalire
 */
public class BasicPoolFactory implements IBasicPoolFactory {

    public <T> IBasicPool<T> createPool(final IPoolItemFactory<T> poolItemFactory) {
        return createBasicPool(poolItemFactory);
    }

    public <E> IBasicPool<E> createBasicPool(final IPoolItemFactory<E> poolItemFactory) {
        return createBasicPool(poolItemFactory, DEFAULT_POOL_SIZE);
    }

    public <E> IBasicPool<E> createBasicPool(final IPoolItemFactory<E> poolItemFactory, final int initPoolSize) {
        return createBasicPool(poolItemFactory, initPoolSize, new LastUsedAccessManager<E>());
    }

    public <E> IBasicPool<E> createBasicPool(final IPoolItemFactory<E> poolItemFactory, final int initPoolSize,
            final IAccessManager<E> releaseListener) {
        return new BasicPool<E>(poolItemFactory, initPoolSize, releaseListener);
    }

}
