/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: BasicCluePoolFactory.java 4759 2009-03-08 19:05:13Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.pool.impl.enhanced.impl.basic.clue;

import org.ow2.util.pool.impl.enhanced.api.IPoolItemFactory;
import org.ow2.util.pool.impl.enhanced.api.basic.clue.IBasicCluePool;
import org.ow2.util.pool.impl.enhanced.api.basic.clue.IBasicCluePoolFactory;
import org.ow2.util.pool.impl.enhanced.api.basic.clue.accessmanager.IClueAccessManager;
import org.ow2.util.pool.impl.enhanced.api.clue.IClueManager;
import org.ow2.util.pool.impl.enhanced.api.clue.basiccluemanager.IClueAccessor;
import org.ow2.util.pool.impl.enhanced.impl.basic.BasicPoolFactory;
import org.ow2.util.pool.impl.enhanced.impl.basic.accessmanager.clue.LastUsedClueAccessManager;
import org.ow2.util.pool.impl.enhanced.impl.clue.basiccluemanager.BasicClueManager;

/**
 * We do not override basic constructor because a pool manager is always needed.
 * @author Gael Lalire
 */
public class BasicCluePoolFactory extends BasicPoolFactory implements IBasicCluePoolFactory {

    public <E extends IClueAccessor<C>, C> IBasicCluePool<E, C> createBasicCluePool(final IPoolItemFactory<E> poolItemFactory) {
        return createBasicCluePool(poolItemFactory, DEFAULT_POOL_SIZE);
    }

    public <E extends IClueAccessor<C>, C> IBasicCluePool<E, C> createBasicCluePool(final IPoolItemFactory<E> poolItemFactory,
            final int initPoolSize) {
        return createBasicCluePool(poolItemFactory, initPoolSize, new BasicClueManager<E, C>());
    }


    public <E, C> IBasicCluePool<E, C> createBasicCluePool(final IPoolItemFactory<E> poolItemFactory,
            final IClueManager<E, C> clueManager) {
        return createBasicCluePool(poolItemFactory, DEFAULT_POOL_SIZE, clueManager);
    }

    public <E, C> IBasicCluePool<E, C> createBasicCluePool(final IPoolItemFactory<E> poolItemFactory, final int initPoolSize,
            final IClueManager<E, C> clueManager) {
        return createBasicCluePool(poolItemFactory, initPoolSize, new LastUsedClueAccessManager<E, C>(clueManager));
    }


    public <E, C> IBasicCluePool<E, C> createCluePool(final IPoolItemFactory<E> poolItemFactory,
            final IClueManager<E, C> clueManager) {
        return createBasicCluePool(poolItemFactory, clueManager);
    }

    public <E, C> IBasicCluePool<E, C> createBasicCluePool(final IPoolItemFactory<E> poolItemFactory, final int initPoolSize,
            final IClueAccessManager<E, C> clueAccessManager) {
        return new BasicCluePool<E, C>(poolItemFactory, initPoolSize, clueAccessManager);
    }

}
