/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: ThreadPool.java 4389 2008-12-15 13:48:57Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.pool.impl.enhanced.impl.thread;

import org.ow2.util.pool.impl.enhanced.api.IPool;
import org.ow2.util.pool.impl.enhanced.api.PoolException;
import org.ow2.util.pool.impl.enhanced.api.basic.IBasicPoolFactory;
import org.ow2.util.pool.impl.enhanced.api.resizer.IResizablePool;
import org.ow2.util.pool.impl.enhanced.api.thread.IReusableThread;
import org.ow2.util.pool.impl.enhanced.impl.basic.BasicPoolFactory;
import org.ow2.util.pool.impl.enhanced.impl.listener.PoolListenerDispatcherPool;
import org.ow2.util.pool.impl.enhanced.impl.listener.resizer.StatsResizerPool;
import org.ow2.util.pool.impl.enhanced.impl.listener.statistics.BasicPoolStats;
import org.ow2.util.pool.impl.enhanced.impl.listener.statistics.BasicPoolStatsFillerPoolListener;
import org.ow2.util.pool.impl.enhanced.impl.thread.managementthread.ManagementThreadReusableThreadFactory;
import org.ow2.util.pool.impl.enhanced.internal.resizer.api.IResizer;
import org.ow2.util.pool.impl.enhanced.internal.resizer.impl.AsynchronousResizer;
import org.ow2.util.pool.impl.enhanced.internal.resizer.impl.stocker.MinMaxExpectedSpareStockerResizer;

/**
 *
 * @author Gael Lalire
 */
public class ThreadPool {

    private static final IBasicPoolFactory DEFAULT_POOL_FACTORY;

    static {
        DEFAULT_POOL_FACTORY = new BasicPoolFactory();
    }

    private PoolListenerDispatcherPool<IReusableThread> pool;

    public ThreadPool() {
        this(false);
    }

    public ThreadPool(final boolean daemonThreadPool) {
        IResizablePool<IReusableThread> resizablePool = DEFAULT_POOL_FACTORY.createBasicPool(new ManagementThreadReusableThreadFactory(
                daemonThreadPool));
        IResizer<BasicPoolStats> resizer = new AsynchronousResizer<BasicPoolStats>(
                new MinMaxExpectedSpareStockerResizer<BasicPoolStats>(10, MinMaxExpectedSpareStockerResizer.NO_MAX_POOL_LIMIT, 5,
                        false));
        resizer.setResizable(resizablePool);
        this.pool = new StatsResizerPool<IReusableThread, BasicPoolStats>(resizablePool,
                new BasicPoolStatsFillerPoolListener<IReusableThread>(), new BasicPoolStats(), resizer);

    }

    public void start(final Runnable runnable) {
        try {
            pool.get(IPool.INFINITE_TIMEOUT).setUsed(pool, runnable);
        } catch (PoolException e) {
            new Error("PoolException should never be thrown by this pool");
        }
    }

}
